package nianio.complib;

import schema.runtime.datatypes.ImmObject;
import nianio.complib.filechooser.FileChooserCommand;
import nianio.ids.UserDialogId;
import schema.runtime.err.SchemaException;
import nianio.complib.messagedialog.MessageDialogCommand;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class DialogCommand = variant {
 *	    fileChooser of FileChooserCommand  
 *	 |  messageDialog of MessageDialogCommand  
 *	 |  userDialog of UserDialogIdCommand  
 * }</pre>
 */ 
public class DialogCommand implements ImmObject {
	private static final int TYPE_HASH = -1385060157;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private DialogCommand(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static DialogCommand createFileChooser(FileChooserCommand value) {
		return new DialogCommand(0, value);
	}

	public static DialogCommand createMessageDialog(MessageDialogCommand value) {
		return new DialogCommand(1, value);
	}

	public static DialogCommand createUserDialog(UserDialogIdCommand value) {
		return new DialogCommand(2, value);
	}


	public static DialogCommand mkUserDialog(
			UserDialogId id
			, UserDialogCommand cmd
	) {
		return createUserDialog( UserDialogIdCommand.create(
							id
							, cmd
							 ));
	}
	

	/** @return true, when this is fileChooser */
	public boolean isFileChooser() {
		return variant == 0;
	}

	/** @return variant value, when this is fileChooser; throws Exception otherwise */
	public FileChooserCommand asFileChooser() {
		SchemaException.ensure(isFileChooser());
		return (FileChooserCommand)value;
	}

	/** @return true, when this is messageDialog */
	public boolean isMessageDialog() {
		return variant == 1;
	}

	/** @return variant value, when this is messageDialog; throws Exception otherwise */
	public MessageDialogCommand asMessageDialog() {
		SchemaException.ensure(isMessageDialog());
		return (MessageDialogCommand)value;
	}

	/** @return true, when this is userDialog */
	public boolean isUserDialog() {
		return variant == 2;
	}

	/** @return variant value, when this is userDialog; throws Exception otherwise */
	public UserDialogIdCommand asUserDialog() {
		SchemaException.ensure(isUserDialog());
		return (UserDialogIdCommand)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isFileChooser()) {
			;
		} else if (isMessageDialog()) {
			;
		} else if (isUserDialog()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isFileChooser()) {
			asFileChooser();
		} else if (isMessageDialog()) {
			asMessageDialog();
		} else if (isUserDialog()) {
			asUserDialog();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isFileChooser())
			return "fileChooser" + "(" + asFileChooser() + ")";
		if (isMessageDialog())
			return "messageDialog" + "(" + asMessageDialog() + ")";
		if (isUserDialog())
			return "userDialog" + "(" + asUserDialog() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(DialogCommand obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof DialogCommand) 
			&& this.eq((DialogCommand)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for DialogCommand
	 */
	public static interface MM {
		public void invoke(DialogCommand cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for DialogCommand
	 */
	public static interface MMIntf {
		public void fileChooser(FileChooserCommand value);
		public void messageDialog(MessageDialogCommand value);
		public void userDialog(
				 UserDialogId id
				, UserDialogCommand cmd
		);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for DialogCommand
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(DialogCommand cmd) {
				 if (cmd.isFileChooser()) {
					mm.fileChooser(cmd.asFileChooser());
				} else if (cmd.isMessageDialog()) {
					mm.messageDialog(cmd.asMessageDialog());
				} else if (cmd.isUserDialog()) {
					UserDialogIdCommand val = cmd.asUserDialog();
					mm.userDialog(
							 val.getId()
							, val.getCmd()
					);
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void fileChooser(FileChooserCommand value) {
				mm.invoke(DialogCommand.createFileChooser(value));
			}
		
			public void messageDialog(MessageDialogCommand value) {
				mm.invoke(DialogCommand.createMessageDialog(value));
			}
		
			public void userDialog(
							 UserDialogId id
							, UserDialogCommand cmd
			) {
				mm.invoke( DialogCommand.createUserDialog(
								UserDialogIdCommand.create( 
									id, cmd)));		
			}
		
		}
	}
}