package nianio.complib.controls;

import nianio.ids.RadioButtonGroupId;
import nianio.ids.ButtonId;
import schema.runtime.datatypes.ImmObject;
import nianio.ids.RadioButtonId;
import nianio.ids.TextBoxId;
import schema.runtime.err.SchemaException;
import ocean.common.Err;
import nianio.ids.LabelId;


/** 
 * <pre>Autogenerated class ControlCommand = variant {
 *	    textBox of TextBoxIdCommand  
 *	 |  label of LabelIdCommand  
 *	 |  button of ButtonIdCommand  
 *	 |  radioButton of RadioButtonIdCommand  
 *	 |  radioButtonGroup of RadioButtonGroupIdCommand  
 * }</pre>
 */ 
public class ControlCommand implements ImmObject {
	private static final int TYPE_HASH = -821064882;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private ControlCommand(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static ControlCommand createTextBox(TextBoxIdCommand value) {
		return new ControlCommand(0, value);
	}

	public static ControlCommand createLabel(LabelIdCommand value) {
		return new ControlCommand(1, value);
	}

	public static ControlCommand createButton(ButtonIdCommand value) {
		return new ControlCommand(2, value);
	}

	public static ControlCommand createRadioButton(RadioButtonIdCommand value) {
		return new ControlCommand(3, value);
	}

	public static ControlCommand createRadioButtonGroup(RadioButtonGroupIdCommand value) {
		return new ControlCommand(4, value);
	}


	public static ControlCommand mkTextBox(
			TextBoxId id
			, TextBoxCommand cmd
	) {
		return createTextBox( TextBoxIdCommand.create(
							id
							, cmd
							 ));
	}
	
	public static ControlCommand mkLabel(
			LabelId id
			, LabelCommand cmd
	) {
		return createLabel( LabelIdCommand.create(
							id
							, cmd
							 ));
	}
	
	public static ControlCommand mkButton(
			ButtonId id
			, ButtonCommand cmd
	) {
		return createButton( ButtonIdCommand.create(
							id
							, cmd
							 ));
	}
	
	public static ControlCommand mkRadioButton(
			RadioButtonId id
			, RadioButtonCommand cmd
	) {
		return createRadioButton( RadioButtonIdCommand.create(
							id
							, cmd
							 ));
	}
	
	public static ControlCommand mkRadioButtonGroup(
			RadioButtonGroupId id
			, RadioButtonGroupCommand cmd
	) {
		return createRadioButtonGroup( RadioButtonGroupIdCommand.create(
							id
							, cmd
							 ));
	}
	

	/** @return true, when this is textBox */
	public boolean isTextBox() {
		return variant == 0;
	}

	/** @return variant value, when this is textBox; throws Exception otherwise */
	public TextBoxIdCommand asTextBox() {
		SchemaException.ensure(isTextBox());
		return (TextBoxIdCommand)value;
	}

	/** @return true, when this is label */
	public boolean isLabel() {
		return variant == 1;
	}

	/** @return variant value, when this is label; throws Exception otherwise */
	public LabelIdCommand asLabel() {
		SchemaException.ensure(isLabel());
		return (LabelIdCommand)value;
	}

	/** @return true, when this is button */
	public boolean isButton() {
		return variant == 2;
	}

	/** @return variant value, when this is button; throws Exception otherwise */
	public ButtonIdCommand asButton() {
		SchemaException.ensure(isButton());
		return (ButtonIdCommand)value;
	}

	/** @return true, when this is radioButton */
	public boolean isRadioButton() {
		return variant == 3;
	}

	/** @return variant value, when this is radioButton; throws Exception otherwise */
	public RadioButtonIdCommand asRadioButton() {
		SchemaException.ensure(isRadioButton());
		return (RadioButtonIdCommand)value;
	}

	/** @return true, when this is radioButtonGroup */
	public boolean isRadioButtonGroup() {
		return variant == 4;
	}

	/** @return variant value, when this is radioButtonGroup; throws Exception otherwise */
	public RadioButtonGroupIdCommand asRadioButtonGroup() {
		SchemaException.ensure(isRadioButtonGroup());
		return (RadioButtonGroupIdCommand)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isTextBox()) {
			;
		} else if (isLabel()) {
			;
		} else if (isButton()) {
			;
		} else if (isRadioButton()) {
			;
		} else if (isRadioButtonGroup()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isTextBox()) {
			asTextBox();
		} else if (isLabel()) {
			asLabel();
		} else if (isButton()) {
			asButton();
		} else if (isRadioButton()) {
			asRadioButton();
		} else if (isRadioButtonGroup()) {
			asRadioButtonGroup();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isTextBox())
			return "textBox" + "(" + asTextBox() + ")";
		if (isLabel())
			return "label" + "(" + asLabel() + ")";
		if (isButton())
			return "button" + "(" + asButton() + ")";
		if (isRadioButton())
			return "radioButton" + "(" + asRadioButton() + ")";
		if (isRadioButtonGroup())
			return "radioButtonGroup" + "(" + asRadioButtonGroup() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(ControlCommand obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof ControlCommand) 
			&& this.eq((ControlCommand)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for ControlCommand
	 */
	public static interface MM {
		public void invoke(ControlCommand cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for ControlCommand
	 */
	public static interface MMIntf {
		public void textBox(
				 TextBoxId id
				, TextBoxCommand cmd
		);
		public void label(
				 LabelId id
				, LabelCommand cmd
		);
		public void button(
				 ButtonId id
				, ButtonCommand cmd
		);
		public void radioButton(
				 RadioButtonId id
				, RadioButtonCommand cmd
		);
		public void radioButtonGroup(
				 RadioButtonGroupId id
				, RadioButtonGroupCommand cmd
		);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for ControlCommand
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(ControlCommand cmd) {
				 if (cmd.isTextBox()) {
					TextBoxIdCommand val = cmd.asTextBox();
					mm.textBox(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isLabel()) {
					LabelIdCommand val = cmd.asLabel();
					mm.label(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isButton()) {
					ButtonIdCommand val = cmd.asButton();
					mm.button(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isRadioButton()) {
					RadioButtonIdCommand val = cmd.asRadioButton();
					mm.radioButton(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isRadioButtonGroup()) {
					RadioButtonGroupIdCommand val = cmd.asRadioButtonGroup();
					mm.radioButtonGroup(
							 val.getId()
							, val.getCmd()
					);
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void textBox(
							 TextBoxId id
							, TextBoxCommand cmd
			) {
				mm.invoke( ControlCommand.createTextBox(
								TextBoxIdCommand.create( 
									id, cmd)));		
			}
		
			public void label(
							 LabelId id
							, LabelCommand cmd
			) {
				mm.invoke( ControlCommand.createLabel(
								LabelIdCommand.create( 
									id, cmd)));		
			}
		
			public void button(
							 ButtonId id
							, ButtonCommand cmd
			) {
				mm.invoke( ControlCommand.createButton(
								ButtonIdCommand.create( 
									id, cmd)));		
			}
		
			public void radioButton(
							 RadioButtonId id
							, RadioButtonCommand cmd
			) {
				mm.invoke( ControlCommand.createRadioButton(
								RadioButtonIdCommand.create( 
									id, cmd)));		
			}
		
			public void radioButtonGroup(
							 RadioButtonGroupId id
							, RadioButtonGroupCommand cmd
			) {
				mm.invoke( ControlCommand.createRadioButtonGroup(
								RadioButtonGroupIdCommand.create( 
									id, cmd)));		
			}
		
		}
	}
}