package nianio.complib.controls;

import nianio.ids.RadioButtonGroupId;
import nianio.ids.ButtonId;
import schema.runtime.datatypes.ImmObject;
import nianio.ids.RadioButtonId;
import nianio.ids.TextBoxId;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class ControlIdEvent = variant {
 *	    buttonEvent of ButtonIdEvent  
 *	 |  radioButtonEvent of RadioButtonIdEvent  
 *	 |  radioButtonGroupIdEvent of RadioButtonGroupIdEvent  
 *	 |  textBoxEvent of TextBoxIdEvent  
 * }</pre>
 */ 
public class ControlIdEvent implements ImmObject {
	private static final int TYPE_HASH = -142599614;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private ControlIdEvent(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static ControlIdEvent createButtonEvent(ButtonIdEvent value) {
		return new ControlIdEvent(0, value);
	}

	public static ControlIdEvent createRadioButtonEvent(RadioButtonIdEvent value) {
		return new ControlIdEvent(1, value);
	}

	public static ControlIdEvent createRadioButtonGroupIdEvent(RadioButtonGroupIdEvent value) {
		return new ControlIdEvent(2, value);
	}

	public static ControlIdEvent createTextBoxEvent(TextBoxIdEvent value) {
		return new ControlIdEvent(3, value);
	}


	public static ControlIdEvent mkButtonEvent(
			ButtonId id
			, ButtonEvent cmd
	) {
		return createButtonEvent( ButtonIdEvent.create(
							id
							, cmd
							 ));
	}
	
	public static ControlIdEvent mkRadioButtonEvent(
			RadioButtonId id
			, RadioButtonEvent cmd
	) {
		return createRadioButtonEvent( RadioButtonIdEvent.create(
							id
							, cmd
							 ));
	}
	
	public static ControlIdEvent mkRadioButtonGroupIdEvent(
			RadioButtonGroupId id
			, RadioButtonGroupEvent cmd
	) {
		return createRadioButtonGroupIdEvent( RadioButtonGroupIdEvent.create(
							id
							, cmd
							 ));
	}
	
	public static ControlIdEvent mkTextBoxEvent(
			TextBoxId id
			, TextBoxEvent cmd
	) {
		return createTextBoxEvent( TextBoxIdEvent.create(
							id
							, cmd
							 ));
	}
	

	/** @return true, when this is buttonEvent */
	public boolean isButtonEvent() {
		return variant == 0;
	}

	/** @return variant value, when this is buttonEvent; throws Exception otherwise */
	public ButtonIdEvent asButtonEvent() {
		SchemaException.ensure(isButtonEvent());
		return (ButtonIdEvent)value;
	}

	/** @return true, when this is radioButtonEvent */
	public boolean isRadioButtonEvent() {
		return variant == 1;
	}

	/** @return variant value, when this is radioButtonEvent; throws Exception otherwise */
	public RadioButtonIdEvent asRadioButtonEvent() {
		SchemaException.ensure(isRadioButtonEvent());
		return (RadioButtonIdEvent)value;
	}

	/** @return true, when this is radioButtonGroupIdEvent */
	public boolean isRadioButtonGroupIdEvent() {
		return variant == 2;
	}

	/** @return variant value, when this is radioButtonGroupIdEvent; throws Exception otherwise */
	public RadioButtonGroupIdEvent asRadioButtonGroupIdEvent() {
		SchemaException.ensure(isRadioButtonGroupIdEvent());
		return (RadioButtonGroupIdEvent)value;
	}

	/** @return true, when this is textBoxEvent */
	public boolean isTextBoxEvent() {
		return variant == 3;
	}

	/** @return variant value, when this is textBoxEvent; throws Exception otherwise */
	public TextBoxIdEvent asTextBoxEvent() {
		SchemaException.ensure(isTextBoxEvent());
		return (TextBoxIdEvent)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isButtonEvent()) {
			;
		} else if (isRadioButtonEvent()) {
			;
		} else if (isRadioButtonGroupIdEvent()) {
			;
		} else if (isTextBoxEvent()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isButtonEvent()) {
			asButtonEvent();
		} else if (isRadioButtonEvent()) {
			asRadioButtonEvent();
		} else if (isRadioButtonGroupIdEvent()) {
			asRadioButtonGroupIdEvent();
		} else if (isTextBoxEvent()) {
			asTextBoxEvent();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isButtonEvent())
			return "buttonEvent" + "(" + asButtonEvent() + ")";
		if (isRadioButtonEvent())
			return "radioButtonEvent" + "(" + asRadioButtonEvent() + ")";
		if (isRadioButtonGroupIdEvent())
			return "radioButtonGroupIdEvent" + "(" + asRadioButtonGroupIdEvent() + ")";
		if (isTextBoxEvent())
			return "textBoxEvent" + "(" + asTextBoxEvent() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(ControlIdEvent obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof ControlIdEvent) 
			&& this.eq((ControlIdEvent)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for ControlIdEvent
	 */
	public static interface MM {
		public void invoke(ControlIdEvent cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for ControlIdEvent
	 */
	public static interface MMIntf {
		public void buttonEvent(
				 ButtonId id
				, ButtonEvent cmd
		);
		public void radioButtonEvent(
				 RadioButtonId id
				, RadioButtonEvent cmd
		);
		public void radioButtonGroupIdEvent(
				 RadioButtonGroupId id
				, RadioButtonGroupEvent cmd
		);
		public void textBoxEvent(
				 TextBoxId id
				, TextBoxEvent cmd
		);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for ControlIdEvent
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(ControlIdEvent cmd) {
				 if (cmd.isButtonEvent()) {
					ButtonIdEvent val = cmd.asButtonEvent();
					mm.buttonEvent(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isRadioButtonEvent()) {
					RadioButtonIdEvent val = cmd.asRadioButtonEvent();
					mm.radioButtonEvent(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isRadioButtonGroupIdEvent()) {
					RadioButtonGroupIdEvent val = cmd.asRadioButtonGroupIdEvent();
					mm.radioButtonGroupIdEvent(
							 val.getId()
							, val.getCmd()
					);
				} else if (cmd.isTextBoxEvent()) {
					TextBoxIdEvent val = cmd.asTextBoxEvent();
					mm.textBoxEvent(
							 val.getId()
							, val.getCmd()
					);
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void buttonEvent(
							 ButtonId id
							, ButtonEvent cmd
			) {
				mm.invoke( ControlIdEvent.createButtonEvent(
								ButtonIdEvent.create( 
									id, cmd)));		
			}
		
			public void radioButtonEvent(
							 RadioButtonId id
							, RadioButtonEvent cmd
			) {
				mm.invoke( ControlIdEvent.createRadioButtonEvent(
								RadioButtonIdEvent.create( 
									id, cmd)));		
			}
		
			public void radioButtonGroupIdEvent(
							 RadioButtonGroupId id
							, RadioButtonGroupEvent cmd
			) {
				mm.invoke( ControlIdEvent.createRadioButtonGroupIdEvent(
								RadioButtonGroupIdEvent.create( 
									id, cmd)));		
			}
		
			public void textBoxEvent(
							 TextBoxId id
							, TextBoxEvent cmd
			) {
				mm.invoke( ControlIdEvent.createTextBoxEvent(
								TextBoxIdEvent.create( 
									id, cmd)));		
			}
		
		}
	}
}