package nianio.ids;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class ControlId = variant {
 *	    panel of PanelId  
 *	 |  radioButton of RadioButtonId  
 *	 |  radioButtonGroup of RadioButtonGroupId  
 *	 |  label of LabelId  
 *	 |  textBox of TextBoxId  
 *	 |  button of ButtonId  
 * }</pre>
 */ 
public class ControlId implements ImmObject {
	private static final int TYPE_HASH = 1608887096;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private ControlId(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static ControlId createPanel(PanelId value) {
		return new ControlId(0, value);
	}

	public static ControlId createRadioButton(RadioButtonId value) {
		return new ControlId(1, value);
	}

	public static ControlId createRadioButtonGroup(RadioButtonGroupId value) {
		return new ControlId(2, value);
	}

	public static ControlId createLabel(LabelId value) {
		return new ControlId(3, value);
	}

	public static ControlId createTextBox(TextBoxId value) {
		return new ControlId(4, value);
	}

	public static ControlId createButton(ButtonId value) {
		return new ControlId(5, value);
	}



	/** @return true, when this is panel */
	public boolean isPanel() {
		return variant == 0;
	}

	/** @return variant value, when this is panel; throws Exception otherwise */
	public PanelId asPanel() {
		SchemaException.ensure(isPanel());
		return (PanelId)value;
	}

	/** @return true, when this is radioButton */
	public boolean isRadioButton() {
		return variant == 1;
	}

	/** @return variant value, when this is radioButton; throws Exception otherwise */
	public RadioButtonId asRadioButton() {
		SchemaException.ensure(isRadioButton());
		return (RadioButtonId)value;
	}

	/** @return true, when this is radioButtonGroup */
	public boolean isRadioButtonGroup() {
		return variant == 2;
	}

	/** @return variant value, when this is radioButtonGroup; throws Exception otherwise */
	public RadioButtonGroupId asRadioButtonGroup() {
		SchemaException.ensure(isRadioButtonGroup());
		return (RadioButtonGroupId)value;
	}

	/** @return true, when this is label */
	public boolean isLabel() {
		return variant == 3;
	}

	/** @return variant value, when this is label; throws Exception otherwise */
	public LabelId asLabel() {
		SchemaException.ensure(isLabel());
		return (LabelId)value;
	}

	/** @return true, when this is textBox */
	public boolean isTextBox() {
		return variant == 4;
	}

	/** @return variant value, when this is textBox; throws Exception otherwise */
	public TextBoxId asTextBox() {
		SchemaException.ensure(isTextBox());
		return (TextBoxId)value;
	}

	/** @return true, when this is button */
	public boolean isButton() {
		return variant == 5;
	}

	/** @return variant value, when this is button; throws Exception otherwise */
	public ButtonId asButton() {
		SchemaException.ensure(isButton());
		return (ButtonId)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isPanel()) {
			;
		} else if (isRadioButton()) {
			;
		} else if (isRadioButtonGroup()) {
			;
		} else if (isLabel()) {
			;
		} else if (isTextBox()) {
			;
		} else if (isButton()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isPanel()) {
			asPanel();
		} else if (isRadioButton()) {
			asRadioButton();
		} else if (isRadioButtonGroup()) {
			asRadioButtonGroup();
		} else if (isLabel()) {
			asLabel();
		} else if (isTextBox()) {
			asTextBox();
		} else if (isButton()) {
			asButton();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isPanel())
			return "panel" + "(" + asPanel() + ")";
		if (isRadioButton())
			return "radioButton" + "(" + asRadioButton() + ")";
		if (isRadioButtonGroup())
			return "radioButtonGroup" + "(" + asRadioButtonGroup() + ")";
		if (isLabel())
			return "label" + "(" + asLabel() + ")";
		if (isTextBox())
			return "textBox" + "(" + asTextBox() + ")";
		if (isButton())
			return "button" + "(" + asButton() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(ControlId obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof ControlId) 
			&& this.eq((ControlId)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	
}